;***************Logic Count Down Game - By Kayleigh Lamb - 000773715 ***************

	;Stack and Stack Pointer Addresses 
.equ	SPH		=$3E	;High Byte Stack Pointer Address 
.equ	SPL		=$3D	;Low Byte Stack Pointer Address 
.equ	RAMEND	=$25F	;Stack Address 

	;Port Addresses 
.equ	PORTB	=$18	;Port B Output Address (7 segment display)
.equ	DDRB	=$17	;Port B Data direction register address

.equ	PORTC	=$15	;Port C Output Address (Buzzer)
.equ	DDRC	=$14	;Port C Data Direction Register Address 

	;Register Definitions
.def	sgmnt	=r0		;Register to store data for seven segment display
.def	temp1	=r16	;Register for temporary storage (load, use and clear)
.def	temp2	=r17	;Register for temporary storage (load, use and clear)
.def	temp3	=r18
.def	address =r19	;Register for table address location
.def	random	=r20	;Register to store random number data
.def	copy	=r21	;Register to store copies of data
.def	count	=r22	

.def    YL		=r28    ;Define low byte of Y 
.def    YH     	=r29    ;Define high byte of Y 
.def	ZL		=r30	;Define low byte of Z 
.def	ZH		=r31	;Define high byte of Z

;Program Initialisation 
	;Set stack pointer to end of memory 
		ldi temp1,high(RAMEND) 
		out SPH,temp1		;Load high byte of end of memory address 
		ldi temp1,low(RAMEND) 
		out SPL,temp1		;Load low byte of end of memory address 
		clr temp1			;reset the temp register

	;Initialise ports for output
		ldi temp1,$FF
		out DDRB,temp1		;Set port B for output
		out	DDRC,temp1		;Set port C for output
		clr temp1			;Reset the temp register


;*************** Begin Main Program ***************
		ldi random,$66		;Load $66 into random number register
main:	rcall rand1			;Call the random number1 subroutine
		rcall hex			;Call the hex subroutine
		rcall buzzer		;Call the buzzer subroutine
		rjmp main			;restart the program

;*************** Random Table Address Number Generator ****************
	;first random number
rand1:	ldi ZL,low(table*2) ;Set Z pointer to start of table 
        ldi ZH,high(table*2)
		ldi	temp1,$1		;load mask to extract bit 0
		ldi	temp2,$2		;load mask to extract bit 1
		and	temp1,random	;Extract bit 0
		and	temp2,random	;Extract bit 1
		lsr	temp2			;shift mask2 right by 1 bit
		eor	temp1,temp2		;XOR bit 1 & 0
		ror	temp1			;Rotate right through carry flag
		ror	random			;Rotate the number in the carry flag onto beg of original number
		clr temp1			;Clear the temp storage registers
		clr temp2

	;Take the random number and ensure it has min - max (0 - 15)
		mov copy,random		;make a copy of the random number
loop:	inc temp1			;increment temp1
		adiw ZL,1			;increment z pointer
		subi copy,$10		;subtract decimal 16 from copy
		brcs minus			;branch if minus
		rjmp loop			;if copy not minus repeat loop
		
minus:	dec temp1
		sbiw ZL,1
		mov address,temp1	;make a copy of the address location for the lookup table
		clr temp1			;clear temp register
		ret					;Return to next line in main program

;**************** Lookup table for 7 segment display hex values ****************

table: .db $3F,$06,$5B,$4F,$66,$6D,$7D,$07,$7F,$6F,$77,$7C,$39,$5E,$79,$71
	
;**************** Send random number to 7 segement display ****************
 
hex:	rcall delay			;Call delay subroutine 
		lpm					;Load sgmnt with data pointed to by Z 
		out PORTB,sgmnt		;and display data on port B 
		rcall delay 
		ret

	;Delay Subroutine (25.349 ms @ 1MHz) 
delay:	ldi YH,high($ffff)	;Load high byte of Y 
        ldi YL,low($ffff)	;Load low byte of Y 
loops:  sbiw Y,1			;Decrement Y 
        brne loops			;and continue to decrement until Y=0 
        ret					;Return to next line 7 segment display routine

;***************Buzzer subroutine 195.84 ms plus 2x 1.911 ms (199.662 ms)***************

buzzer: ldi	temp2,$FF		;Initialise loop2 counter 
two:  	ldi	temp1,$FF		;Initialise loop1 counter 
		ldi	temp3,$FF		;Load temp3 with 1's
		out	PORTC,temp3		;Load port C with 1 on bit 3
		rcall period		;Period of sound wave
		ldi	temp3,$00		;Load temp3 with 0's
		out	PORTC,temp3		;Load port C with 0 on bit 3
		rcall period		;period of sound wave
one:  	dec	temp1           ;Decrement loop one counter 
        brne one			;If loop one counter not yet 0, go round loop1 again 
        dec temp2			;Decrement loop two counter 
        brne two			;If loop two counter not yet 0, go round loop2 again 
        ret					;Return to main program

	;Period delay section of code (1.911 ms @ 1MHz) - utilises r25 & r24

period:  ldi r24,$19		;Initialise 2nd loop B counter 
loop2:   ldi r25,$19		;Initialise 1st loop A counter 
loop1:   dec r25			;Decrement the 1st loop counter 
         brne loop1			;and continue to decrement until 1st loop counter = 0 
         dec r24			;Decrement the 2nd loop counter 
         brne loop2			;If the 2nd loop counter is not equal to zero repeat the 1st loop, else continue 
         ret 

