;***************Logic Count Down Game - By Kayleigh Lamb - 000773715 ***************

	;Stack and Stack Pointer Addresses 
.equ	SPH		=$3E	;High Byte Stack Pointer Address 
.equ	SPL		=$3D	;Low Byte Stack Pointer Address 
.equ	RAMEND	=$25F	;Stack Address 

	;Port Addresses 
.equ	PORTB	=$18	;Port B Output Address (7 segment display)
.equ	DDRB	=$17	;Port B Data direction register address

.equ	PORTC	=$15	;Port C Output Address (Buzzer)
.equ	DDRC	=$14	;Port C Data Direction Register Address 

	;Register Definitions
.def 	sgmnt	=r0
.def	temp	=r16
.def	slave	=r17
.def	adhoc	=r18
.def	master	=r19
.def	store	=r20

.def	random1 =r21
.def	random2	=r22	

.def    YL		=r28    ;Define low byte of Y 
.def    YH     	=r29    ;Define high byte of Y 
.def	ZL		=r30	;Define low byte of Z 
.def	ZH		=r31	;Define high byte of Z

;Program Initialisation 
	;Set stack pointer to end of memory (uses temp)
		ldi temp,high(RAMEND) 
		out SPH,temp		;Load high byte of end of memory address 
		ldi temp,low(RAMEND) 
		out SPL,temp		;Load low byte of end of memory address 
		clr temp	

	;Initialise ports for output (uses temp)
		ldi temp,$FF
		out DDRB,temp		;Set port B for output
		out	DDRC,temp		;Set port C for output
		clr temp
		
;*************** Begin Main Program ***************
		ldi random1,$66		;Load $66 into random number register
		ldi random2,$1f
main:	rcall leds			;Call the random number subroutine
		rcall buzzer		;Call the buzzer subroutine
		rjmp main

;***************Buzzer subroutine 195.84 ms plus 2x 1.911 ms***************

buzzer: ldi	temp,$FF		;Initialise loop2 counter 
two:  	ldi	adhoc,$FF		;Initialise loop1 counter 
		ldi	slave,$FF		;Load slave with $FF
		out	PORTC,slave		;Load port C with $FF (1 on bit 3 needed)
		rcall period		;Period of sound wave
		ldi	slave,$00		;Load Slave with $00
		out	PORTC,slave		;Load port C with 0 on bit 3
		rcall period		;period of sound wave
one:  	dec	adhoc           ;Decrement loop one counter 
        brne one            ;If loop one counter not yet 0, go round loop1 again 
        dec temp           	;Decrement loop two counter 
        brne two            ;If loop two counter not yet 0, go round loop2 again 
        ret					;Return to main program

	;Period delay section of code (1.911 ms @ 1MHz) - utilises r25 & r24

period:  ldi    r24,$19        	;Initialise 2nd loop B counter 
loopt:   ldi    r25,$19        ;Initialise 1st loop A counter 
loopo:   dec    r25             ;Decrement the 1st loop counter 
         brne   loopo           ;and continue to decrement until 1st loop counter = 0 
         dec    r24             ;Decrement the 2nd loop counter 
         brne   loopt          ;If the 2nd loop counter is not equal to zero repeat the 1st loop, else continue 
         ret 

;**************** Generate random hex number and send to the 7 segment display**************
	
	;Generate the first random hex number 
		ldi temp,$01		;Load mask to extract bit 0
		ldi	adhoc,$02		;Load mask to extract bit 1
		and	temp,random1	;Extract bit 0
		and	adhoc,random1	;Extract bit 1
		lsr adhoc			;shift right 1 bit
		eor	temp,adhoc		;XOR bit 1 & 0
		ror temp			;Rotate right through carry flag
		ror random1			;Rotate the number in the carry flafg onto the beg of original number
		clr temp			;Clear temp registers
		clr adhoc

	;Generate the second random number	
		ldi temp,$01		;Load mask to extract bit 0
		ldi	adhoc,$02		;Load mask to extract bit 1
		and	temp,random2	;Extract bit 0
		and	adhoc,random2	;Extract bit 1
		lsr adhoc			;shift right 1 bit
		eor	temp,adhoc		;XOR bit 1 & 0
		ror temp			;Rotate right through carry flag
		ror random2			;Rotate the number in the carry flag onto the beg of original number
		clr temp			;Clear temp registers
		clr adhoc

	;Ensure first random number has a min - max (0 - F)

leds:	ldi ZL,low(table*2)	;set the Z pointer to start of table
		ldi	ZH,high(table*2)
loop2:	mov temp,random1	;make a temp copy of random1
loop1:	ldi	master,$FF		;Initialise 1st loop
		ldi slave,$FF		;Initialise 2nd loop
subt:	adiw ZL,1			;Increment the z pointer#
		subi temp,$10		;Subtract $10 from temp
		brcs minus			;Branch if temp is minus
		rjmp subt			;else keep subtracting
	
	;Send first number to port B
minus:	sbiw ZL,1			;Subtract one to even out
		lpm					;Load sgmnt with data pointed to by z
		rcall delay
		out PORTB,sgmnt		;Send data to port B
		clr temp			;Clear temp register
	
	;Ensure second random number has a min - max (0 - F)
		ldi ZL,low(table*2)	;set the Z pointer to start of table
		ldi	ZH,high(table*2)
		mov temp,random2	;make a temp copy of random2
subtr:	adiw ZL,1			;Increment the z pointer#
		subi temp,$10		;Subtract $10 from temp
		brcs nega			;Branch if temp is negative
		rjmp subtr			;else keep subtracting

	;Send send second number to port B
nega:	sbiw ZL,1			;Subtract one to even out
		lpm					;Load sgmnt with data pointed to by z
		rcall delay
		out PORTB,sgmnt		;Send data to port B
		clr temp			;Clear temp register
		dec master			;Decrement the first loop
		brne loop1			;and continue until first loop = 0
		dec slave			;Decrement the second loop
		brne loop2
		ret					;Return to the next line in the main program

	;Delay Subroutine (25.349 ms @ 1MHz) 
delay:   ldi    YH,high($ffff)    ;Load high byte of Y 
         ldi    YL,low($ffff)     ;Load low byte of Y 
loops:   sbiw   Y,1               ;Decrement Y 
         brne   loops             ;and continue to decrement until Y=0 
         ret

;**************** Lookup table for 7 segment display hex values ****************

table: .db $3F,$06,$5B,$4F,$66,$6D,$7D,$07,$7F,$6F,$77,$7C,$39,$5E,$79,$71		
